#!/usr/bin/perl -w
#
#  Copyright 2000, G. Wade Johnson	
#  Released under the same license as Perl.
#
#  Version: 0.7.1
#
#  Validate a PLL file or run regression tests on PLL.pm
#

use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/lib";
use PLL;
use XML::Parser;
use XML::Data::DataDoc::Style;
use strict;

my $p = new XML::Parser( Style=>'XML::Data::DataDoc::Style' );

my $rebuild = 0;
my $testfile= "$FindBin::Bin/TestPLL.tests";

if(@ARGV and '-rebuild' eq $ARGV[0])
 {
  shift;
  $rebuild = 1;
 }

if(@ARGV)
 {
  while(@ARGV)
   {
    my $file = shift;

    my $success = TestPLL( $file );
    print "$file is a valid PLL file.\n" if $success;
   }
 }
elsif($rebuild)
 {
  local $/ = '';
  my $input;
  my $errs;
  my $tmpfile = "$testfile~";

  unlink( $tmpfile ) if -f $tmpfile;
  open( TESTS, $testfile ) or die "Unable to open tests file.\n";
  open( OUT, ">$tmpfile" ) or die "Unable to create output tests file.";

  while($input = <TESTS>)
   {
    my ($result,$test) = (undef,'unnamed');
    last if $input =~ /^\s*$/s;
    
    $result = $1 if $input =~ s/# err:([^\n]*\n)//sm;
    $test = $1   if $input =~ s/# ([^\n]*)\n//sm;

    $result = $@, $errs++ unless(TestPLLstring( $input, $result ));
    
    print OUT "# $test\n";
    print OUT "# err:$result" if $result;
    print OUT "$input";
   }

  close( OUT );
  close( TESTS );

  if($errs)
   {
    print "Corrections made to tests file.\n";
    unlink( $testfile ) or die "Unable to delete old tests file \'$tmpfile'\n";
    rename( $tmpfile, $testfile ) or die "Unable to rename tests file \'$testfile'\n";
   }
  else
   {
    print "No corrections needed.\n";
    unlink( $tmpfile ) or die "Unable to delete temporary file \'$tmpfile'\n";
   }
 }
else # regression test
 {
  local $/ = '';
  my $input;
  my $errs;
  my $successes;

  open( TESTS, $testfile ) or die "Unable to open tests file.\n";

  while($input = <TESTS>)
   {
    my ($result,$test) = (undef,'unnamed');
    last if $input =~ /^\s*$/s;
    
    $result = $1 if $input =~ s/# err:([^\n]*\n)//sm;
    $test = $1   if $input =~ s/# ([^\n]*)\n//sm;

    if(TestPLLstring( $input, $result ))
     {
      ++$successes;
     }
    else
     {
      ++$errs, print "Test \'$test\' failed.\n";
     }
   }

  close( TESTS );

  if($errs)
   {
    print "$errs tests failed out of ", $errs+$successes, ".\n";
   }
  else
   {
    print "$successes tests successful.\n";
   }
 }

sub  TestPLL
 {
  my $file = shift;
  local $/ = undef;

  open( F, $file ) or die "Unable to open $file.\n";
  my $input = <F>;
  close( F ) or die "Unable to close $file.\n";
  
  TestPLLstring( $input );
 }

sub  TestPLLstring
 {
  my $str    = shift;
  my $result = shift || '';

  my $dd = $p->parse( $str );
  
  my $pll = $dd->Root();
  die "Unable to parse input string.\n" unless defined $pll;

  eval { PLL::validate( $pll, undef ); };

  print $@ if $@ ne $result;
  
  $@ eq $result;
 }
