#!/usr/bin/perl -w
#
#  Copyright 2000, G. Wade Johnson
#  Released under the same license as Perl.
#
#  Version: 0.7.1
#
#  Validate a WSDL file or run regression tests on WSDL.pm
#

# ToDo List
#   1. validate enumerated attributes

use FindBin;
use lib "$FindBin::Bin/lib";
use WSDL;
use Data::Dumper;
use Getopt::Std;
use strict;




#--------------------------------------------
#  Begin main code

my $rebuild = 0;
my $testfile= "$FindBin::Bin/WSDLvalid.tests";

if(@ARGV and '-rebuild' eq $ARGV[0])
 {
  shift;
  $rebuild = 1;
 }

my %CmdLine = ();
extract_defines();

if(@ARGV)
 {
  while(@ARGV)
   {
    my $file = shift;

    my $wsdl = new WSDL( $file, %CmdLine );

    $wsdl->validate();
    
    print "WSDL file $file verified.\n";
   }
 }
elsif($rebuild)
 {
  local $/ = '';
  my $input;
  my $errs;
  my $tmpfile = "$testfile~";

  unlink( $tmpfile ) if -f $tmpfile;
  open( TESTS, $testfile ) or die "Unable to open tests file.\n";
  open( OUT, ">$tmpfile" ) or die "Unable to create output tests file.";

  while($input = <TESTS>)
   {
    my ($result,$test) = (undef,'unnamed');
    last if $input =~ /^\s*$/s;
    
    $result = $1 if $input =~ s/# err:([^\n]*\n)//sm;
    $test = $1   if $input =~ s/# ([^\n]*)\n//sm;

    $result = $@, $errs++ unless(testWSDLstring( $input, $result ));
    
    print OUT "# $test\n";
    print OUT "# err:$result" if $result;
    print OUT "$input";
   }

  close( OUT );
  close( TESTS );

  if($errs)
   {
    print "Corrections made to tests file.\n";
    unlink( $testfile ) or die "Unable to delete old tests file \'$tmpfile'\n";
    rename( $tmpfile, $testfile ) or die "Unable to rename tests file \'$testfile'\n";
   }
  else
   {
    print "No corrections needed.\n";
    unlink( $tmpfile ) or die "Unable to delete temporary file \'$tmpfile'\n";
   }
 }
else # regression test
 {
  local $/ = '';
  my $input;
  my $errs;
  my $successes;

  open( TESTS, $testfile ) or die "Unable to open tests file.\n";

  while($input = <TESTS>)
   {
    my ($result,$test) = (undef,'unnamed');
    last if $input =~ /^\s*$/s;
    
    $result = $1 if $input =~ s/# err:([^\n]*\n)//sm;
    $test = $1   if $input =~ s/# ([^\n]*)\n//sm;

    if(testWSDLstring( $input, $result ))
     {
      ++$successes;
     }
    else
     {
      ++$errs, print "Test \'$test\' failed.\n";
     }
   }

  close( TESTS );

  if($errs)
   {
    print "$errs tests failed out of ", $errs+$successes, ".\n";
   }
  else
   {
    print "$successes tests successful.\n";
   }
 }

#---------------------------------------------
#  Subroutines

sub  extract_defines
 {
  while(@ARGV)
   {
    last unless $ARGV[0] =~ /(\w+)=(.*)/;
    
    $CmdLine{$1} = $2;
    shift @ARGV;
   }
 }


sub  testWSDLstring
 {
  my $input  = shift;
  my $result = shift || '';
  
  my $wsdl = new WSDL( undef, %CmdLine );
  $wsdl->parse( $input );
    
  eval { $wsdl->validate() };
  
  print $@ if $@ ne $result;
  
  $@ eq $result;
 }


